`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Engineer:       Michael Siegenthaler
//
// Design Name:    Dual-port RAM
// Module Name:    ioram
// Revision:       July 16, 2005
// 
////////////////////////////////////////////////////////////////////////////////
module ioram(clk_a, addr_a, en_a, we_a, di_a, do_a,
             clk_b, addr_b, en_b, we_b, di_b, do_b);

  // Specify RAM size
  parameter RAM_WIDTH = 8;
  parameter RAM_ADDR_BITS = 8;

  // Inputs/outputs
  input clk_a;
  input [RAM_ADDR_BITS-1:0] addr_a;
  input en_a;
  input we_a;
  input [RAM_WIDTH-1:0] di_a;
  output [RAM_WIDTH-1:0] do_a;
  input clk_b;
  input [RAM_ADDR_BITS-1:0] addr_b;
  input en_b;
  input we_b;
  input [RAM_WIDTH-1:0] di_b;
  output [RAM_WIDTH-1:0] do_b;

  // External nodes
  wire clk_a;
  wire [RAM_ADDR_BITS-1:0] addr_a;
  wire en_a;
  wire we_a;
  wire [RAM_WIDTH-1:0] di_a;
  wire [RAM_WIDTH-1:0] do_a;
  wire clk_b;
  wire [RAM_ADDR_BITS-1:0] addr_b;
  wire en_b;
  wire we_b;
  wire [RAM_WIDTH-1:0] di_b;
  wire [RAM_WIDTH-1:0] do_b;

  // Internal nodes
  wire [8:0] addr_big_a, addr_big_b;
  assign addr_big_a[RAM_WIDTH-1:0] = addr_a;
  assign addr_big_a[8] = 1'b0;
  assign addr_big_b[RAM_WIDTH-1:0] = addr_b;
  assign addr_big_b[8] = 1'b0;

  // Instantiate a 256x16bit RAM block
  RAMB4_S8_S8 ram0 (
    .CLKA(clk_a),
    .ENA(en_a),
    .WEA(we_a),
    .RSTA(1'b0),
    .ADDRA(addr_big_a),
    .DIA(di_a),
    .DOA(do_a),
    .CLKB(clk_b),
    .ENB(en_b),
    .WEB(we_b),
    .RSTB(1'b0),
    .ADDRB(addr_big_b),
    .DIB(di_b),
    .DOB(do_b) );

endmodule
